@@ -3,7 +3,7 @@ package Chess::Play;
 use strict;
 use warnings;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use constant IL => 99;
 use constant EM => 0;
@@ -52,6 +52,7 @@ sub new {
 	$self->{PIECE_VAL} = {};
 	$self->{DEPTH} = undef;
 	$self->{COLOR_TO_MOVE} = undef;
+	$self->{FEN_MOVE_NUMBER} = undef;
 
 	bless ($self, $class);
 	return $self;
@@ -96,6 +97,206 @@ sub reset {
 	$self->{PIECE_VAL}{+WQ} = $self->{PIECE_VAL}{+BQ} = Q_VAL;
 
 	$self->{COLOR_TO_MOVE} = WHITE;
+
+	$self->{FEN_MOVE_NUMBER} = 1;
+}
+
+sub import_fen {
+	my ($self, $fen) = @_;
+	my ($r, $c, $sq);
+	my ($fen_1, $fen_2, $fen_3, $fen_4, $fen_5, $fen_6);
+
+	#reset board to illegal
+	for (my $i = 0; $i < 144; $i++) {
+		$self->{BOARD}[$i] = IL;
+	}
+
+	my @fen_arr = split(/ /, $fen);
+	die "Invalid FEN" if (@fen_arr != 6);
+
+	# pieces' position
+	$fen_1 = $fen_arr[0];
+	my @fen_rows = split(/\//, $fen_1);
+	die "Invalid FEN - first element wrong" if (@fen_rows != 8);
+
+	while ($fen_1 =~ /(\d)/g) {
+		my $num = $1;
+		my $rep = " " x $num;
+		$fen_1 =~ s/$num/$rep/;
+	}
+	$fen_1 =~ s/\///g;
+
+	my $fen_i = 0;
+	for ($r = 7; $r >= 0; $r--) {
+		for ($c = 0; $c <= 7; $c++) {
+			$sq = 12 * ($r + 2) + 2 + $c;
+			my $fen_piece = substr($fen_1, $fen_i, 1);
+			$self->{BOARD}[$sq] = fen_to_board($fen_piece);
+			$fen_i++;
+		}
+	}
+
+	# color to move
+	$fen_2 = $fen_arr[1];
+	if ($fen_2 eq "w") {
+		$self->{COLOR_TO_MOVE} = WHITE;
+	}
+	elsif ($fen_2 eq "b") {
+		$self->{COLOR_TO_MOVE} = BLACK;
+	}
+	else {
+		die "Invalid FEN - second element wrong";
+	}
+
+	# castle flags
+	$fen_3 = $fen_arr[2];
+	if ($fen_3 =~ /K/) {
+		$self->{CASTLE_OK}{E1G1} = 1;
+	}
+	else {
+		$self->{CASTLE_OK}{E1G1} = 0;
+	}
+	if ($fen_3 =~ /Q/) {
+		$self->{CASTLE_OK}{E1C1} = 1;
+	}
+	else {
+		$self->{CASTLE_OK}{E1C1} = 0;
+	}
+	if ($fen_3 =~ /k/) {
+		$self->{CASTLE_OK}{E8G8} = 1;
+	}
+	else {
+		$self->{CASTLE_OK}{E8G8} = 0;
+	}
+	if ($fen_3 =~ /q/) {
+		$self->{CASTLE_OK}{E8C8} = 1;
+	}
+	else {
+		$self->{CASTLE_OK}{E8C8} = 0;
+	}
+
+	# en passant
+	$self->{LAST_DOUBLE_MOVE} = [ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 ];
+	$fen_4 = $fen_arr[3];
+	if ($fen_4 =~ /([a-h])3/) {				# double pawn move for white
+		my $column = ord($1)-97;
+		$self->{LAST_DOUBLE_MOVE}[$column] = 1;
+	}
+	elsif ($fen_4 =~ /([a-h])6/) {				# double pawn move for black
+		my $column = ord($1)-97;
+		$self->{LAST_DOUBLE_MOVE}[8+$column] = 1;
+	}
+
+	# halfmove clock
+	$fen_5 = $fen_arr[4];
+	die "Invalid FEN - fifth element wrong" if (not ($fen_5 =~ /\d/));
+	$self->{RULE_50_MOVES} = $fen_5;
+
+	# fullmove clock
+	$fen_6 = $fen_arr[5];
+	$self->{FEN_MOVE_NUMBER} = $fen_6;
+
+	# check flags
+	if ($self->can_capture_king(-$self->{COLOR_TO_MOVE})) {
+		if ($self->{COLOR_TO_MOVE} == WHITE) {
+			$self->{UNDER_CHECK}{W_K} = 1;
+			$self->{UNDER_CHECK}{B_K} = 0;
+		}
+		else {
+			$self->{UNDER_CHECK}{W_K} = 0;
+			$self->{UNDER_CHECK}{B_K} = 1;
+		}
+	}
+	else {
+		$self->{UNDER_CHECK}{B_K} = 0;
+		$self->{UNDER_CHECK}{W_K} = 0;
+	}
+
+	# sad necessity
+	$self->{PIECE_VAL}{+WP} = $self->{PIECE_VAL}{+BP} = P_VAL;
+	$self->{PIECE_VAL}{+WN} = $self->{PIECE_VAL}{+BN} = N_VAL;
+	$self->{PIECE_VAL}{+WB} = $self->{PIECE_VAL}{+BB} = B_VAL;
+	$self->{PIECE_VAL}{+WR} = $self->{PIECE_VAL}{+BR} = R_VAL;
+	$self->{PIECE_VAL}{+WQ} = $self->{PIECE_VAL}{+BQ} = Q_VAL;
+}
+
+sub export_fen {
+	my $self = shift;
+	my ($r, $c, $sq);
+	my ($fen_1, $fen_2, $fen_3, $fen_4, $fen_5, $fen_6, $fen);
+
+	$fen_1 = "";
+	for ($r = 7; $r >= 0; $r--) {
+		for ($c = 0; $c <= 7; $c++) {
+			$sq = 12 * ($r + 2) + 2 + $c;
+			my $str = value_to_string($self->{BOARD}[$sq]);
+			$fen_1 = $fen_1 . $str;
+		}
+		$fen_1 = $fen_1 . '/';
+	}
+	$fen_1 =~ s/\/$//;
+	while ($fen_1 =~ /( +)/g) {
+		my $spaces = $1;
+		my $nb_spaces = length($spaces);
+
+		$fen_1 =~ s/$spaces/$nb_spaces/;
+	}
+
+	if ($self->{COLOR_TO_MOVE} == WHITE) {
+		$fen_2 = "w";
+	}
+	else {
+		$fen_2 = "b";
+	}
+
+	$fen_3 = "";
+	$fen_3 = $fen_3 . "K" if ($self->{CASTLE_OK}{E1G1});
+	$fen_3 = $fen_3 . "Q" if ($self->{CASTLE_OK}{E1C1});
+	$fen_3 = $fen_3 . "k" if ($self->{CASTLE_OK}{E8G8});
+	$fen_3 = $fen_3 . "q" if ($self->{CASTLE_OK}{E8C8});
+	$fen_3 = "-" if (not $fen_3);
+
+	$fen_4 = "-";
+	for (my $ldm = 0; $ldm < 16; $ldm++) {
+		if ($self->{LAST_DOUBLE_MOVE}[$ldm]) {
+			if ($ldm < 8) {
+				$fen_4 = chr(97+$ldm) . "3";
+			}
+			else {
+				$fen_4 = chr(97+$ldm-8) . "6";
+			}
+			last;
+		}
+	}
+
+	$fen_5 = $self->{RULE_50_MOVES};
+
+	$fen_6 = $self->{FEN_MOVE_NUMBER};
+
+	$fen = "$fen_1 $fen_2 $fen_3 $fen_4 $fen_5 $fen_6";
+	return $fen;
+}
+
+sub fen_to_board {
+	my $fen_piece = shift;
+
+	return EM if ($fen_piece eq " ");
+
+	return WR if ($fen_piece eq "R");
+	return WN if ($fen_piece eq "N");
+	return WB if ($fen_piece eq "B");
+	return WQ if ($fen_piece eq "Q");
+	return WK if ($fen_piece eq "K");
+	return WP if ($fen_piece eq "P");
+
+	return BR if ($fen_piece eq "r");
+	return BN if ($fen_piece eq "n");
+	return BB if ($fen_piece eq "b");
+	return BQ if ($fen_piece eq "q");
+	return BK if ($fen_piece eq "k");
+	return BP if ($fen_piece eq "p");
+
+	die "Wrong value in FEN strin : $fen_piece";
 }
 
 sub set_piece_val {
@@ -198,6 +399,20 @@ sub best_move {
 	return move_to_coord($bestmove);
 }
 
+sub print_board {
+	my $self = shift;
+
+	my ($r, $c, $sq);
+	for ($r = 7; $r >= 0; $r--) {
+		for ($c = 0; $c <= 7; $c++) {
+			$sq = 12 * ($r + 2) + 2 + $c;
+			my $str = value_to_string($self->{BOARD}[$sq]);
+			print "$str ";
+		}
+		print "\n";
+	}
+}
+
 sub play {
 	my $self = shift;
 
@@ -335,20 +550,6 @@ sub rule50moves {
 	return 0;
 }
 
-sub print_board {
-	my $self = shift;
-
-	my ($r, $c, $sq);
-	for ($r = 7; $r >= 0; $r--) {
-		for ($c = 0; $c <= 7; $c++) {
-			$sq = 12 * ($r + 2) + 2 + $c;
-			my $str = value_to_string($self->{BOARD}[$sq]);
-			print "$str ";
-		}
-		print "\n";
-	}
-}
-
 sub knight_mvs {
 	my $self = shift;
 
@@ -814,6 +1015,8 @@ sub execute_move {
 	my $moving_piece = $self->{BOARD}[$orig_square];
 	my $moving_color = sign($moving_piece);
 
+	$self->{FEN_MOVE_NUMBER}++ if ($moving_color == BLACK);
+
 	# capture or pawn move
 	if ( ($self->{BOARD}[$dest_square] != EM) or ($moving_piece == WP) or ($moving_piece == BP) ) {
 		$self->{RULE_50_MOVES} = 0;
@@ -992,6 +1195,7 @@ sub test_legality {
 	my %saved_under_check = %{ $self->{UNDER_CHECK} };
 	my $saved_rule_50_moves = $self->{RULE_50_MOVES};
 	my $saved_color_to_move = $self->{COLOR_TO_MOVE};
+	my $saved_move_number = $self->{FEN_MOVE_NUMBER};
 
 	# execute move
 	$self->execute_move($move);
@@ -1008,6 +1212,7 @@ sub test_legality {
 	%{ $self->{UNDER_CHECK} } = %saved_under_check;
 	$self->{RULE_50_MOVES} = $saved_rule_50_moves;
 	$self->{COLOR_TO_MOVE} = $saved_color_to_move;
+	$self->{FEN_MOVE_NUMBER} = $saved_move_number;
 
 	return $is_legal;
 }
@@ -1096,6 +1301,7 @@ sub alphabeta_search {
 			my %saved_under_check = %{ $self->{UNDER_CHECK} };
 			my $saved_rule_50_moves = $self->{RULE_50_MOVES};
 			my $saved_color_to_move = $self->{COLOR_TO_MOVE};
+			my $saved_move_number = $self->{FEN_MOVE_NUMBER};
 
 			#shuffle @legal_moves array
 			fisher_yates_shuffle(\@legal_moves) if ($depth == $self->{DEPTH});
@@ -1112,6 +1318,7 @@ sub alphabeta_search {
 				%{ $self->{UNDER_CHECK} } = %saved_under_check;
 				$self->{RULE_50_MOVES} = $saved_rule_50_moves;
 				$self->{COLOR_TO_MOVE} = $saved_color_to_move;
+				$self->{FEN_MOVE_NUMBER} = $saved_move_number;
 
 				if ($evaluation >= $beta) {
 					return $beta;
@@ -1145,6 +1352,7 @@ sub alphabeta_search {
 			my %saved_under_check = %{ $self->{UNDER_CHECK} };
 			my $saved_rule_50_moves = $self->{RULE_50_MOVES};
 			my $saved_color_to_move = $self->{COLOR_TO_MOVE};
+			my $saved_move_number = $self->{FEN_MOVE_NUMBER};
 
 			#shuffle @legal_moves array
 			fisher_yates_shuffle(\@legal_moves) if ($depth == $self->{DEPTH});
@@ -1160,6 +1368,7 @@ sub alphabeta_search {
 				%{ $self->{UNDER_CHECK} } = %saved_under_check;
 				$self->{RULE_50_MOVES} = $saved_rule_50_moves;
 				$self->{COLOR_TO_MOVE} = $saved_color_to_move;
+				$self->{FEN_MOVE_NUMBER} = $saved_move_number;
 
 				if ($evaluation >= $beta) {
 					return $beta;
@@ -1453,6 +1662,8 @@ Chess::Play - Play chess games, calculate legal moves, use a search algorithm
 
   my $cp = Chess::Play->new();
   $cp->reset();
+  $cp->import_fen($fen)
+  $cp->export_fen()
   $cp->set_piece_val($p_val, $n_val, $b_val, $r_val, $q_val);
   $cp->set_depth($depth)
   $cp->legal_moves()
@@ -1480,6 +1691,14 @@ Create a new object to play chess.
 
 Reset to the start position.
 
+=item * $cp->import_fen($fen)
+
+Set the position according to the FEN string $fen
+
+=item * $fen = $cp->export_fen()
+
+Export the current position to the FEN string $fen
+
 =item * $cp->set_piece_val($p_val, $n_val, $b_val, $r_val, $q_val)
 
 Change default values for pieces (the defaults are : 1, 3, 3, 5, 9)
@@ -1518,28 +1737,36 @@ Play a chess game using STDIN
 
 Play a chess game using xboard (only the basic xboad directives are supported). You can choose a name for your engine.
 
- Examples:
+=head1 EXAMPLES
+
+=item * Create a new Chess::Play object
   my $cp = Chess::Play->new();
 
-  Execute some moves
+=item * Execute some moves
   if ($cp->do_move("e2e4") == 1) {..}
   if ($cp->do_move("e7e5") == 1) {..}
 
-  Calculating legal moves
-  my @legal_moves = $cp->legal_moves()
+=item * Calculate legal moves
+  @legal_moves = $cp->legal_moves()
 
-  Play a chess game using stdin
+=item * Play a chess game using stdin (using coordinate notation)
   $cp->reset();
   $ce->set_depth(2);
   $ce->play();
 
-  Play a chess game using xboard
+=item * Create an xboard-compatible chess engine
   $cp->reset();
   $ce->set_depth(2);
   $cp->xboard_play("My_Chess_Engine")
 
+=item * Use xboard
   xboard -fcp /path/to/my_engine.pl or xboard -fcp /path/to/my_engine.pl -scp /path/to/my_engine2.pl
 
+=item * Find a mate in 2
+  $cp->import_fen("1R6/8/KN6/8/1k6/7r/2QP4/8 w - - 0 1");
+  $cp->set_depth(3);
+  $bm = $cp->best_move();
+
 =back
 
 =head1 AUTHOR
